<?php
define(OBJECT_PATH, dirname(__FILE__));

define(OBJECT_TYPE_ARRAY,   'Array');
define(OBJECT_TYPE_STRING,  'String');
define(OBJECT_TYPE_URL,     'Url');
define(OBJECT_TYPE_INTEGER, 'Integer');

/**
 * Class Object_Abstract
 *
 * @author   Anton Shevchuk <AntonShevchuk@gmail.com>
 * @link     http://anton.shevchuk.name
 * @access   public
 * @category Object
 * @package  Object_Abstract
 * @created  Tue Oct 28 11:51:43 EET 2008
 */
abstract class Object_Abstract
{    
    /**
     * String
     *
     * @var mixed
     */
    protected $var;
    
    
    /**
     * Constructor of oObject
     *
     * @access  public
     * @param   array $var
     * @return  void
     */
    public function __construct($var) 
    {
        $this->var = $var;
    }
     
     /**
      * __toString
      *
      * return string
      *
      * @class   oObject
      * @access  public
      * @return  mixed
      */
     public function __toString() 
     {
         return (string)$this->var;
     }
     
     /**
      * __call
      *
      * universal wrapper
      *
      * @access  public
      * @param   string   $method
      * @param   array    $params
      * @return  Object_Abstract
      */
     public function __call($method, $params) 
     {
         if (function_exists($method)) {
             $this->var = call_user_func_array($method, $params);
             return $this;
         } else {
             throw new Exception('Method "' . $method . '" is not exist');
         }
     }
     
    /**
     * __get
     *
     * it's magic O_o
     *
     * @access  public
     * @param   string   $method
     * @return  mixed   
     */
    public function __get($method) 
    {
        if (method_exists($this, $method)) {
            return $this->$method();
        } else {
            return $this;
        }
    }
     
     /**
      * __params
      *
      *
      * @class   oObject
      * @access  protected
      * @param   array    $reflection  array of reflection params
      * @param   array    $needle      array of "true" name of $this->var
      * @param   array    $params      
      * @return  array    params with included "$this->var"
      */
     protected function __params($reflection, $needle, $params) 
     {
          $pos = 0;
          $size = sizeof($reflection);
          
          if ($size == 0) {
              return array($this->var);
          }
          
          // search params name
         for($i = 0; $i < $size; $i++) {
             if (in_array($reflection[$i]->name, $needle)) {
                 $pos = $i;
                 break;
             }
         }
         
         // insert $this->var to params array
         if ($pos == 0) {
             array_unshift($params, $this->var);
         } elseif ($pos == $size) {
             array_push($params, $this->var);
         } else {
             $ar1 = array_slice($params, 0, $pos);
             $ar2 = array_slice($params, -($pos-1));
             
             $params   = $ar1;
             $params[] = $this->var;
             
             $params   = array_merge($params, $ar2);
         }
         
         return $params;
     }
     
     /**
      * get
      *
      * return a varibale
      *
      * @class   oObject
      * @access  public
      * @return  mixed  
      */
     public function get() 
     {
         return $this->var;
     }
     
     /**
      * size
      *
      * @access  public
      * @return  integer
      */
     abstract public function size();
     
     /**
      * zero
      *
      * check empty or not
      *
      * @access  public
      * @return  bool
      */
     public function zero() 
     {
         return $this->size()==0;
     }
}