<?php
/**
 * Class Object_Array
 *
 * object model for array
 *
 * @author   Anton Shevchuk <AntonShevchuk@gmail.com>
 * @link     http://anton.shevchuk.name
 * @access   public
 * @category Object
 * @package  Object_Array
 * @created  Tue Oct 28 10:57:25 EET 2008
 */
require_once 'Abstract.php';

class Object_Array extends Object_Abstract
{
    /**
     * Type 
     *
     * @var string
     */
    protected $type = OBJECT_TYPE_ARRAY;
    
    /**
     * __call
     *
     * magic method
     *
     * @author  dark
     * @class   oString
     * @access  public
     * @param   string     $function  function name
     * @param   array      $params    function params
     * @return  Object_Array
     */
    public function __call($function, $params) 
    {
        if (!function_exists($function)) {
            $func = 'array_'.$function;
            
            if (!function_exists($func)) {
                throw new Exception('Function "' . $function . '" is not exist');
            }
            
            $function = $func;
        }
        
        $func = new ReflectionFunction($function);

        $needle = array('subject', 'haystack', 'input');
        
        $params = $this -> __params($func -> getParameters(), $needle, $params);

        $this->var = call_user_func_array($function, $params);
        return $this;
    }
    
    /**
     * __get
     *
     * return a array element
     *
     * @access  public
     * @param   mixed     $index  array index
     * @return  mixed   
     */
    public function __get($index) 
    {
        if (isset($this->var[$index])) {
            return $this->var[$index];
        } else {
            return false;
        }
    }
    
    /**
     * merge
     *
     * merge wrapper
     *
     * @class   oArray
     * @access  public
     * @param   mixed    $array
     * @return  Object_Array
     */
    public function merge($array) 
    {
        if (func_num_args() != 1) {
            $array = func_get_args();
        } else {
            $array = array($array);
        }
        
        array_unshift($array, $this->var);
        
        $this->var = call_user_func_array('array_merge', $array);
        return $this;
    }
    
    /**
     * combine
     *
     * array_combine wrapper
     *
     * @access  public
     * @param   array     $array
     * @return  Object_Array
     */
    public function combine($array) 
    {
        if (is_a($array, 'Object_Array')) {
            $array = $array->get();
        }
        
        $this->var = array_combine($this->var, $array);
        if (!$this->var) {
            throw new Exception("Number of elements for each array isn't equal");
        }
        return $this;
    }
    
    /**
     * implode
     *
     * implode wrapper
     *
     * @access  public
     * @param   string   $string
     * @return  Object_String
     */
    public function implode($string) 
    {
        return oString(implode($string, $this->var));
    }
    
    /**
     * range
     *
     * range wrappper
     *
     * @access  public
     * @param   integer  $start
     * @param   integer  $stop
     * @param   integer  $step
     * @return  Object_Array
     */
    public function range($start, $stop, $step = 1) 
    {
        $this->var = range($start, $stop, $step);        
        return $this;
    }
    
    /**
     * reverse
     *
     * reverse array
     *
     * @access  public
     * @param   bool     $key  preserver key
     * @return  Object_Array
     */
    public function reverse($key = false) 
    {
        $this->var = array_reverse($this->var, $key);
        return $this;
    }
    
    /**
     * sort
     *
     * sort array
     *
     * @access  public
     * @param   integer $flag
     * @return  Object_Array  
     */
    public function sort($flag = SORT_REGULAR) 
    {
        sort($this->var, SORT_REGULAR);
        return $this;
    }
    
    /**
     * rsort
     *
     * rsort array
     *
     * @access  public
     * @param   integer $flag
     * @return  Object_Array  
     */
    public function rsort($flag = SORT_REGULAR) 
    {
        rsort($this->var, SORT_REGULAR);
        return $this;
    }
    
    /**
     * natsort
     *
     * natsort array
     *
     * @access  public
     * @return  Object_Array  
     */
    public function natsort() 
    {
        natsort($this->var);
        return $this;
    }
    
    /**
     * natcasesort 
     *
     * rsort array
     *
     * @access  public
     * @return  Object_Array  
     */
    public function natcasesort () 
    {
        natcasesort($this->var);
        return $this;
    }
    
    /**
     * ksort
     *
     * ksort array
     *
     * @access  public
     * @param   integer $flag
     * @return  Object_Array  
     */
    public function ksort($flag = SORT_REGULAR) 
    {
        ksort($this->var, SORT_REGULAR);
        return $this;
    }
    
    /**
     * krsort
     *
     * krsort array
     *
     * @access  public
     * @param   integer $flag
     * @return  Object_Array  
     */
    public function krsort($flag = SORT_REGULAR) 
    {
        krsort($this->var, SORT_REGULAR);
        return $this;
    }
        
    /**
     * to_url
     *
     * http_build_query wrapper
     *
     * @access  public
     * @param   string   $prefix
     * @param   string   
     * @return  Object_Url
     */
    public function to_url($prefix = null, $separator = '&') 
    {
        return oUrl(http_build_query($this->var, $prefix, $separator));
    }
    
    /**
     * first
     *
     * return first element
     *
     * @access  public
     * @return  mixed
     */
    public function first() 
    {
        $first = array_shift($this->var);
        array_unshift($this->var, $first);
        
        return $first;
    }
    
    /**
     * last
     *
     * return last element
     *
     * @access  public
     * @return  mixed
     */
    public function last() 
    {
        $lastt = array_pop($this->var);
        array_push($this->var, $last);
        
        return $last;
    }
    
    /**
     * clear
     *
     * remove empty elements
     *
     * @access  public
     * @param   type     $param  param_descr
     * @return  Object_Array
     */
    public function clear() 
    {
        $res = array();
        foreach ($this->var as $key => $val)  {
            if (!empty($val)) {
                if (is_numeric($key)) {
                    array_push($res, $val);
                } else {
                    $res[$key] = $val;
                }
            }
        }
        $this->var = $res;
        return $this;
    }
    
    /**
     * Odd method
     *
     * @param   integer $var
     * @return  Object_Array
     */
    public function odd() {
        $this->var = array_filter($this->var, array($this, "_odd"));
        return $this;
        
    }
    
    /**
     * Callback function for odd method
     *
     * @param  integer $var
     * @return bool
     */
    private function _odd($var) {
        return ($var % 2 == 1);
    }

    /**
     * Even method
     *
     * @param   integer $var
     * @return  Object_Array
     */
    public function even() {
        $this->var = array_filter($this->var, array($this, "_even"));
        return $this;
    }
    
    /**
     * Callback function for even method
     *
     * @param integer $var
     * @return bool
     */
    private function _even($var) {
        return ($var % 2 == 0);
    }

    /**
     * size
     *
     * return array size
     *
     * @access  public
     * @return  integer
     */
    public function size() 
    {
//         return oInteger(sizeof($this->var));
        return sizeof($this->var);
    }
}

