<?php
/**
 * Class Object_String
 *
 * object model for string
 *
 * @author   Anton Shevchuk <AntonShevchuk@gmail.com>
 * @link     http://anton.shevchuk.name
 * @access   public
 * @category Object
 * @package  Object_String
 * @created  Tue Oct 28 10:57:25 EET 2008
 */
require_once 'Abstract.php';

class Object_String extends Object_Abstract
{
	/**
	 * __call
	 *
	 * magic method
	 *
	 * @access  public
	 * @param   string     $function  function name
	 * @param   array      $params    function params
	 * @return  Object_String
	 */
	public function __call($function, $params) 
	{
	    if (!function_exists($function)) {
	    	$func = 'str_'.$function;
	    	
	    	if (!function_exists($func)) {
	    		$func = 'str'.$function;
	    	}
	    	
	    	if (!function_exists($func)) {
	    		throw new Exception('Function "' . $function . '" is not exist');
	    	}
	    	
	    	$function = $func;
	    }
	    
	    $func = new ReflectionFunction($function);

	    $needle = array('subject', 'haystack', 'input');
	    
	    $params = $this -> __params($func -> getParameters(), $needle, $params);

	    $this->var = call_user_func_array($function, $params);
	    return $this;
	}
	
	/**
     * pad
     *
     * str_pad wrapper
     *
     * @access  public
     * @param   integer    $length  
     * @param   string     $string  
     * @param   integer    $type  
     * @return  Object_String
     */
    public function pad($length, $string = ' ', $type = STR_PAD_RIGHT) 
    {
    	$this->var = str_pad($this->var, $length, $string, $type);
    	return $this;
    }
    
	/**
     * sub
     *
     * substr wrapper
     *
     * @access  public
     * @param   integer    $start
     * @param   integer    $type  
     * @return  Object_String
     */
    public function sub($start, $length = null) 
    {
    	$this->var = substr($this->var,$start, $length);
    	return $this;
    }
    
    /**
     * repeat
     *
     * str_repeat wrapper
     *
     * @access  public
     * @param   integer    $length  
     * @return  Object_String
     */
    public function repeat($length) 
    {
    	$this->var = str_repeat($this->var, $length);
    	return $this;
    }
        
    /**
     * shuffle
     *
     * str_shuffle wrapper
     *
     * @access  public
     * @return  Object_String
     */
    public function shuffle() 
    {
    	$this->var = str_shuffle($this->var);
    	return $this;
    }
    
	 /**
	  * replace
	  *
	  * str_replace wrapper
	  *
	  * @access  public
	  * @param   mixed     $search
	  * @param   mixed     $replace
	  * @param   integer   $count
	  * @return  Object_String
	  */
	 public function replace($search, $replace, &$count = null) 
	 {
	     $this->var = str_replace($search, $replace, $this->var, &$count);
	 	 return $this;
	 }
	 
	 /**
	  * ireplace
	  *
	  * str_ireplace wrapper
	  *
	  * @access  public
	  * @param   mixed     $search
	  * @param   mixed     $replace
	  * @param   integer   $count
	  * @return  Object_String
	  */
	 public function ireplace($search, $replace, &$count = null) 
	 {
	     $this->var = str_ireplace($search, $replace, $this->var, &$count);
	 	 return $this;
	 }
	 
	 /**
	  * nl2br 
	  *
	  * nl2br wrapper
	  *
	  * @access  public
	  * @return  Object_String
	  */
	 public function nl2br() 
	 {
	 	$this->var = nl2br($this->var);
	 	return $this;
	 }
	     
    /**
     * word_count
     *
     * str_word_count wrapper
     *
     * @access  public
     * @param   integer $type
     * @param   string  $charlist
     * @return  Object_String | Object_Array
     */
    public function word_count($type = 0, $charlist = '') 
    {
        // switch statement for $type
        switch ($type) {
            case 1:
            case 2:
                return oArray($this->var, $type, $charlist);
                break;
        	case 0:
        	default:        	    
    	        $this->var = str_word_count($this->var, $type, $charlist);
        		break;
        }
    	return $this;
    }
    
    /**
     * word_wrap
     *
     * wardwrap wrapper
     *
     * @access  public
     * @param   integer  $width
     * @param   integer  $break
     * @param   bool     $cut
     * @return  Object_String
     */
    public function word_wrap($width = null, $break = null, $cut = null) 
    {
        $this->var = wordwrap($this->var, $width, $break, $cut);
        return $this;
    }
    
    
    /**
     * html_encode
     *
     * htmlentities wrapper
     *
     * @access  public
     * @param   integer $style
     * @param   string  $charset
     * @return  Object_String
     */
    public function html_encode($style = ENT_COMPAT, $charset = 'ISO-8859-1') 
    {
        $this->var = htmlentities($this->var, $style, $charset);
        return $this;
    }
    
    /**
     * html_decode
     *
     * html_entity_decode wrapper
     *
     * @access  public
     * @param   integer $style
     * @param   string  $charset
     * @return  Object_String
     */
    public function html_decode($style = ENT_COMPAT, $charset = 'ISO-8859-1') 
    {
        $this->var = html_entity_decode($this->var, $style, $charset);
        return $this;
    }
    
    /**
     * html_specialchars_encode
     *
     * htmlspecialchars wrapper
     *
     * @access  public
     * @param   integer $style
     * @param   string  $charset
     * @return  Object_String
     */
    public function html_specialchars_encode($style = ENT_COMPAT, $charset = 'ISO-8859-1') 
    {
        $this->var = htmlspecialchars($this->var, $style, $charset);
        return $this;
    }
    
    /**
     * html_specialchars_decode
     *
     * htmlspecialchars_decode wrapper
     *
     * @access  public
     * @param   integer $style
     * @return  Object_String
     */
    public function html_specialchars_decode($style = ENT_COMPAT) 
    {
        $this->var = htmlspecialchars_decode($this->var, $style);
        return $this;
    }
    
	 /**
	  * split
	  *
	  * split wrapper
	  *
	  * @access  public
	  * @param   string     $pattern
	  * @param   integer    $limit
	  * @return  Object_Array
	  */
	 public function split($pattern, $limit = null) 
	 {
	    if ($limit) {
	       $this->var = split($pattern, $this->var, $limit);
	    } else {
	       $this->var = split($pattern, $this->var);	        
	    }
	    return oArray($this->var);
	 }
	 
	 /**
	  * explode
	  *
	  * explode wrapper
	  *
	  * @access  public
	  * @param   string     $delimiter
	  * @param   integer    $limit
	  * @return  Object_Array
	  */
	 public function explode($delimiter, $limit = null) 
	 {
	    if ($limit) {
	       $this->var = explode($delimiter, $this->var, $limit);
	    } else {
	       $this->var = explode($delimiter, $this->var);	        
	    }
	    return oArray($this->var);
	 }
     
     /**
      * strrev
      *
      * reverse string
      *
      * @access  public
      * @return  Object_String
      */
     public function reverse() 
     {
         $this->var = strrev($this->var);
         return $this;
     }
     
     
     /**
      * add
      *
      * add string to current
      *
      * @access  public
      * @param   string $string
      * @return  Object_String
      */
     public function add($string) 
     {
         $this->var .= $string;
         return $this;
     }
          
     /**
      * insert
      *
      * insert string to string
      *
      * @access  public
      * @param   string   $text
      * @param   integer  $position
      * @return  Object_String
      */
     public function insert($text, $position = 0) 
     {
         $this->var = substr_replace($this->var, $text, $position, 0);
         return $this;
     }
     
     /**
      * size
      *
      * return string length
      *
      * @access  public
      * @return  integer
      */
     public function size() 
     {
//         return oInteger(strlen($this->var));
         return strlen($this->var);
     }
}